@@ -1,4 +1,4 @@
-use serde_json::{json, Value};
+use bson::{bson, Bson, to_bson};
 use sqlx_core::column::Column;
 use sqlx_core::decode::Decode;
 use sqlx_core::error::BoxDynError;
@@ -15,47 +15,48 @@ use sqlx_core::value::ValueRef;
 
 use crate::convert::{JsonCodec, RefJsonCodec, ResultCodec};
 use crate::postgres::PgInterval;
-use chrono::Utc;
+use chrono::{Local, Utc};
 
-use crate::{new_json_option_into};
+use crate::{to_bson_macro};
 use std::option::Option::Some;
-use crate::types::DateStringCheck;
+use bson::spec::BinarySubtype;
+use crate::db::db_adapter::DataDecoder;
 
 impl<'c> JsonCodec for PgValueRef<'c> {
-    fn try_to_json(self) -> crate::Result<serde_json::Value> {
+    fn try_to_bson(self) -> crate::Result<Bson> {
         match self.type_info().name() {
             "VOID" => {
-                return Ok(serde_json::Value::Null);
+                return Ok(Bson::Null);
             }
             "NUMERIC" => {
                 //decimal
                 let r: Option<BigDecimal> = Decode::<'_, Postgres>::decode(self)?;
                 if let Some(date) = r {
-                    return Ok(serde_json::Value::String(date.to_string()));
+                    return Ok(Bson::String(date.to_string()));
                 }
-                return Ok(serde_json::Value::Null);
+                return Ok(Bson::Null);
             }
             "NUMERIC[]" => {
                 //decimal
                 let r: Option<Vec<BigDecimal>> = Decode::<'_, Postgres>::decode(self)?;
                 if let Some(r) = r {
-                    return Ok(serde_json::json!(r));
+                    return Ok(to_bson(&r).unwrap_or_default());
                 }
-                return Ok(serde_json::Value::Null);
+                return Ok(Bson::Null);
             }
             "MONEY" => {
                 //decimal
                 let r: Option<String> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "MONEY[]" => {
                 //decimal
                 let r: Option<Vec<String>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "BOOL" => {
                 let r: Option<bool> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "BOOL[]" => {
                 let r: Vec<bool> = Decode::<'_, Postgres>::decode(self)?;
@@ -64,93 +65,100 @@ impl<'c> JsonCodec for PgValueRef<'c> {
             "BYTEA" | "BYTEA[]" => {
                 let r: Option<Vec<u8>> = Decode::<'_, Postgres>::decode(self)?;
                 if let Some(r) = r {
-                    return Ok(serde_json::json!(crate::types::byte::RbBytes{
-                        rb_bytes:r,
+                    return Ok(Bson::Binary(bson::Binary {
+                        subtype: BinarySubtype::Generic,
+                        bytes: r,
                     }));
                 }
-                return Ok(serde_json::Value::Null);
+                return Ok(Bson::Null);
             }
             "FLOAT4" => {
                 let r: Option<f32> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "FLOAT4[]" => {
                 let r: Option<Vec<f32>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "FLOAT8" => {
                 let r: Option<f64> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "FLOAT8[]" => {
                 let r: Option<Vec<f64>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "INT2" => {
                 let r: Option<i16> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                if let Some(r) = r {
+                    return Ok(bson!(r as i32));
+                }
+                return Ok(Bson::Null);
             }
             "INT2[]" => {
                 let r: Option<Vec<i16>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                if let Some(r) = r {
+                    return Ok(to_bson(&r).unwrap_or_default());
+                }
+                return Ok(Bson::Null);
             }
             "INT4" => {
                 let r: Option<i32> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "INT4[]" => {
                 let r: Option<Vec<i32>> = Decode::<'_, Postgres>::decode(self)?;
 
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "INT8" => {
                 let r: Option<i64> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "INT8[]" => {
                 let r: Option<Vec<i64>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "OID" => {
                 let r: Option<u32> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "OID[]" => {
                 let r: Option<Vec<u32>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "TEXT" | "NAME" | "VARCHAR" | "BPCHAR" | "CHAR" | "\"CHAR\"" | "UNKNOWN" => {
                 let r: Option<String> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "TEXT[]" | "CHAR[]" | "VARCHAR[]" | "\"CHAR\"[]" | "NAME[]" => {
                 let r: Option<Vec<String>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                return Ok(to_bson_macro!(r));
             }
             "UUID" => {
                 let r: Option<Uuid> = Decode::<'_, Postgres>::decode(self)?;
                 if let Some(r) = r {
-                    return Ok(serde_json::Value::String(r.to_string()));
+                    return Ok(to_bson(&crate::types::Uuid::from(r)).unwrap_or_default());
                 }
-                return Ok(serde_json::Value::Null);
+                return Ok(Bson::Null);
             }
             "UUID[]" => {
                 let r: Option<Vec<Uuid>> = Decode::<'_, Postgres>::decode(self)?;
                 if let Some(r) = r {
-                    let mut arr =vec![];
+                    let mut arr = vec![];
                     for x in r {
-                        arr.push(x.to_string());
+                        arr.push(to_bson(&crate::types::Uuid::from(x)).unwrap_or_default());
                     }
-                    return Ok(serde_json::Value::from(arr));
+                    return Ok(Bson::from(arr));
                 }
-                return Ok(serde_json::Value::Null);
+                return Ok(Bson::Null);
             }
             "JSON" | "JSONB" => {
                 let r: Option<Json<serde_json::Value>> = Decode::<'_, Postgres>::decode(self)?;
                 if let Some(r) = r {
-                    return Ok(r.0);
+                    return Ok(to_bson(&r.0).unwrap_or_default());
                 }
-                return Ok(serde_json::Value::Null);
+                return Ok(Bson::Null);
             }
             "JSON[]" | "JSONB[]" => {
                 let r: Option<Vec<Json<serde_json::Value>>> = Decode::<'_, Postgres>::decode(self)?;
@@ -159,66 +167,90 @@ impl<'c> JsonCodec for PgValueRef<'c> {
                     for x in r {
                         arr.push(x.0);
                     }
-                    return Ok(serde_json::Value::Array(arr));
+                    return Ok(to_bson(&arr).unwrap_or_default());
                 }
-                return Ok(serde_json::Value::Null);
+                return Ok(Bson::Null);
             }
             "TIME" => {
-                let r: Option<Time> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                let r: Option<NaiveTime> = Decode::<'_, Postgres>::decode(self)?;
+                return Ok(to_bson(&r).unwrap_or_default());
             }
             "TIME[]" => {
-                let r: Option<Vec<Time>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                let r: Option<Vec<NaiveTime>> = Decode::<'_, Postgres>::decode(self)?;
+                return Ok(to_bson(&r).unwrap_or_default());
             }
             "DATE" => {
                 let r: Option<chrono::NaiveDate> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                return Ok(to_bson(&r).unwrap_or_default());
             }
             "DATE[]" => {
                 let r: Option<Vec<chrono::NaiveDate>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                return Ok(to_bson(&r).unwrap_or_default());
             }
             "TIMESTAMP" => {
-                let r: Option<chrono::NaiveDateTime> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                let r: Option<chrono::DateTime<Local>> = Decode::<'_, Postgres>::decode(self)?;
+                if let Some(r) = r {
+                    let dt = bson::DateTime::from_chrono(r);
+                    return Ok(Bson::DateTime(dt));
+                }
+                return Ok(Bson::Null);
             }
             "TIMESTAMP[]" => {
-                let r: Option<Vec<chrono::NaiveDateTime>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                let r: Option<Vec<chrono::DateTime<Local>>> = Decode::<'_, Postgres>::decode(self)?;
+                if let Some(r) = r {
+                    let mut dts = vec![];
+                    for x in r {
+                        let dt = bson::DateTime::from_chrono(x);
+                        dts.push(Bson::DateTime(dt));
+                    }
+                    return Ok(Bson::Array(dts));
+                }
+                return Ok(Bson::Null);
             }
             "TIMESTAMPTZ" => {
                 let r: Option<chrono::DateTime<Utc>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                if let Some(r) = r {
+                    let dt = bson::DateTime::from_chrono(r);
+                    return Ok(Bson::DateTime(dt));
+                }
+                return Ok(Bson::Null);
             }
             "TIMESTAMPTZ[]" => {
                 let r: Option<Vec<chrono::DateTime<Utc>>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                if let Some(r) = r {
+                    let mut dts = vec![];
+                    for x in r {
+                        let dt = bson::DateTime::from_chrono(x);
+                        dts.push(Bson::DateTime(dt));
+                    }
+                    return Ok(Bson::Array(dts));
+                }
+                return Ok(Bson::Null);
             }
             "CIDR" | "INET" => {
                 let r: Option<IpNetwork> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                return Ok(to_bson(&r).unwrap_or_default());
             }
             "CIDR[]" | "INET[]" => {
                 let r: Option<Vec<IpNetwork>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                return Ok(to_bson(&r).unwrap_or_default());
             }
 
             "INTERVAL" => {
                 let r: Option<sqlx_core::postgres::types::PgInterval> =
                     Decode::<'_, Postgres>::decode(self)?;
                 if r.is_none() {
-                    return Ok(serde_json::Value::Null);
+                    return Ok(Bson::Null);
                 }
-                return Ok(json!(PgInterval::from(r.unwrap())));
+                return Ok(to_bson(&PgInterval::from(r.unwrap())).unwrap_or_default());
             }
             "VARBIT" | "BIT" => {
                 let r: Option<bit_vec::BitVec> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                return Ok(to_bson(&r).unwrap_or_default());
             }
             "VARBIT[]" | "BIT[]" => {
                 let r: Option<Vec<bit_vec::BitVec>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(json!(r));
+                return Ok(to_bson(&r).unwrap_or_default());
             }
             _ => {
                 //TODO
@@ -243,25 +275,33 @@ impl<'c> JsonCodec for PgValueRef<'c> {
                 // "MACADDR","MACADDR[]",
                 //  you can use already Vec<u8> types to decode this
                 let r: Option<Vec<u8>> = Decode::<'_, Postgres>::decode(self)?;
-                return Ok(new_json_option_into!(r));
+                if let Some(r) = r {
+                    return Ok(Bson::Binary(bson::Binary {
+                        subtype: BinarySubtype::Generic,
+                        bytes: r,
+                    }));
+                }
+                return Ok(Bson::Null);
             }
         }
     }
 }
 
 impl RefJsonCodec for Vec<PgRow> {
-    fn try_to_json(&self) -> crate::Result<serde_json::Value> {
+    fn try_to_bson(&self, decoder: &dyn DataDecoder) -> crate::Result<Bson> {
         let mut arr = Vec::with_capacity(self.len());
         for row in self {
-            let mut m = serde_json::Map::new();
+            let mut m = bson::Document::new();
             let columns = row.columns();
             for x in columns {
                 let key = x.name();
                 let v: PgValueRef = row.try_get_raw(key)?;
-                m.insert(key.to_owned(), v.try_to_json()?);
+                let mut bson=v.try_to_bson()?;
+                decoder.decode(key,&mut bson)?;
+                m.insert(key.to_owned(), bson);
             }
-            arr.push(serde_json::Value::Object(m));
+            arr.push(Bson::Document(m));
         }
-        Ok(serde_json::Value::from(arr))
+        Ok(Bson::from(arr))
     }
 }
